#!/bin/bash
# Call to get pictures and movies from Campark T85 camera
# author timm@ganske.org 2020ff

usage[ 0]="usage: T85get.sh (T85cmd) T85destdir T85nasdir T85camWLAN T85stdWLAN T85camWLANPWD T85stdWLANPWD"
usage[ 1]="    if you did not modify the CAM WLAN settings, giving the first two parameters is sufficient"
usage[ 2]="    if they are set in the environment, parameters are not needed"
usage[ 4]=" "
usage[ 5]="T85cmd may be:"
usage[ 6]="  -r repeat    repeat until files exist in T85nasdir (default cmd)"
usage[ 7]="  -1 get once  one try to get files from the camera"
usage[ 8]="  -f output format string sample data and length"
usage[10]="  -h help      usage"
usage[11]="(only parameters starting with '-' will be treated as command, unknown commands give usage)"
usage[12]=" "
usage[13]="  debug output to screen: export T85DBG=enable"
usage[14]="  debug output to file  : export T85LOGDBG=enable"


# Overview T85get scripts
#
#* T85 get files
#
# T85get.sh       -- do all the work, fetch files from camera using Campark WLAN
#                 function T85log          -- do logging
#                 function T85eval_key     -- react on keypress
#                 function T85hmtlloop.sh  -- iterate over the file list from camera webserver
#                 function T85getdir       -- fetch a directory from T85 cam (eg PHOTO or MOVIES)
# T85enablewlan.sh-- enables WLAN using Bluetooth
# T85setwlan.sh   -- switch Wifi to some other SSID - NUL as WLAN for no action
# T85getfile.sh   -- fetch one file from camera, remove on success
#
#* T85 exif rename
#
# T85exifren.sh   -- rename files and move them according to exifdata, using exiftool
# T85exifwait.sh  -- wait for appearing files, then start T85exifren
#
#* T85 general purpose
#
# T85move.sh      -- replacement for mv, make sure the file is safe and binary identical

# environment variables that may be predefined
#
# T85destdir        -- fetch the files from the camera to this place
# T85nasdir         -- where to move the files after fetching and re-enable home WLAN
# T85camWLAN        -- the name of the Campark T85 WLAN, substring is sufficient (first matching WLAN will be taken)
# T85stdWLAN        -- the name of the std WLAN to connect after fetching files
# T85stdWLANturnoff -- set this to any string to have WLAN turned off after usage
# T85routeprohibit  -- do not do route add for the WLAN
# T85camWLANPWD     -- password for T85camWLAN (default 12345678)
# T85stdWLANPWD     -- password for T85stdWLAN (default empty, sufficient if password is stored or shall be asked)
# T85getshdir       -- directory for the T85 get scripts
# T85exifrencall    -- if T85get.sh shall call exifren too, the call to be done is defined here

# tweak parameters
#
# T85DBG            -- if set, produces much more output, usually containing dbg, mainly tty only
# T85LOGDBG         -- if set, writes progress to logfile, automatically enables T85DBG
# T85DBGfile        -- if set, writes all html Files to file system for analyzing
# T85DBGdir         -- dir to write all html Files to file system for analyzing
# T85loglevel       -- Which parts to debug. If not set, everything is logged if T85DBG is set
# T85getlog         -- name of logfile, default "T85get.log"
# T85minfilesize    -- files smaller than this size get  ignored, default 10240
# T85maxfetch       -- stop fetching files if T85maxfetch files transferred
# T85maxseqfail     -- stop fetching files after T85maxseqfail errors in sequence without succesful transmit
# T85maxfail        -- stop fetching files after T85maxfail errors on transmit
# T85getrepeatsleep -- time between connection tries in T85getrepeat.sh
# T85curltimeout    -- options seconds for curl --connection-timeout parameter
# T85userentrytime  -- time for question to user e. g. "files exist, continue anyway?"
# T85progressarr    -- Name of array how to output progress.
# T85progress       -- Default Array how to output progress. T85progress[0] is checked if predefined values exist.
# T85idledefault    -- idle-delay to set after run of fetch
# T85idlerun        -- idle-delay to set for run of fetch
# T85idlegetcmd     -- if T85idledefault is not set, fetch it using this command
# T85idlesetcmd     -- How to set T85idledefault
# T85inhibitsleep   -- How to call T85get.sh -1 preventing sleep

# parameter especially used by T85enablewlan.sh
# T85GATTTIMEOUT    -- Timeout for sending bluetooth command to en/disable WLAN
# T85GATTSLEEPAFTER -- Sleep time after send WLAN on/off command to T85 Bluetooth
# T85BLUEMACADDR    -- You may give a MAC address, then it won't be detected
# T85BLUESEARCH     -- The search string how the device name is given by bluetoothctl default "^Device 62:00:"

# parameter especially used by T85setwlan.sh
# T85WSCANSLEEPAFTER-- Sleep after Rescan WLAN

# not necessarily intended to be set by anyone, but will change behavior
# T85camurl         -- URL to access, default http://192.168.8.120
# T85curlopt        -- options for curl, default "-s -S" for debug, otherwise empty
# T85delmatch       -- Default "?del=1" to search for file URL, will not work for T85 if set different
# T85delstring      -- Default $T85delmatch, may return complete file to output if set different

#                 -- Note on mv:
#                 -- Please note that the delayed write of SMB on Synology NAS may cause that
#                 -- your data is not safe. There may be several pictures written only to RAM
#                 -- and lost if power outage occurs. In my case if would be about 17 files of
#                 -- 1 MB that get written to RAM inside my DS218+ and the original deleted
#                 -- before any disk inside the NAS starts spinning.
#                 -- The verify of T85move does not help against a write cache, so T85get.sh
#                 -- currently only uses mv, not T85move.sh. This is probably subject to change
#                 -- if a solution for this problem is available.

# If first parameter starts with '-', it is a command
if [[ "${1:0:1}" == "-" ]]; then
  T85cmd="$1"
  shift
else
  T85cmd='-r'
fi

# Parameter that can be given, override defaults from script/environment
[ "$1" ] && T85destdir="$1"
[ "$2" ] &&  T85nasdir="$2"
[ "$3" ] && T85camWLAN="$3"
[ "$4" ] && T85stdWLAN="$4"
[ "$5" ] && T85camWLANPWD="$5"
[ "$6" ] && T85stdWLANPWD="$6"

# These will be executed by eval. Injection into used variables should be not possible,
# as they are not entered externally.
# Guess assuming the camera does not put commands or apostrophes inside
# the filename is fine.
# Commands can be set by environment, so be careful who may set your environment variables before
# calling this skript.
#
# If changed, make sure that the parameter for printf is set
if [[ ${#T85progress[@]} = 0 ]]; then
   T85progress[ 00]='printf "$now"'
   T85progress[ 10]='printf " === %3i %% ===" $percent'
#  T85progress[ 20]='printf " %4is" $secsdiff'
   T85progress[ 30]='printf " %5ims" $msecsdiff'
#  T85progress[ 40]='printf " %5is" $secsused'
#  T85progress[ 50]='printf " %5is" $secseta'
   T85progress[ 60]='printf " %5i" $ifetched'
   T85progress[ 70]='echo -n "/"'
   T85progress[ 80]='printf "%5i" $htmlfilecount'
   T85progress[ 90]='printf " -  "'
   T85progress[100]='printf "%15s" ":$filename:"'
   T85progress[110]='printf " ==="'
   T85progress[120]='echo -n "  - $finishtime"'
   T85progress[130]='echo -n " $finishtimeglobal"'
   T85progress[990]='printf "\n"'
fi

if [[ ${#T85progress_m[@]} = 0 ]]; then
#  T85progress_m[ 00]='printf "$now"'
   T85progress_m[ 10]='printf " === %3i %% ===" $percent'
   T85progress_m[ 20]='printf " %4is" $secsdiff'
#  T85progress_m[ 30]='printf " %5ims" $msecsdiff'
#  T85progress_m[ 40]='printf " %5is" $secsused'
#  T85progress_m[ 50]='printf " %5is" $secseta'
   T85progress_m[ 60]='printf " %5i" $ifetched'
   T85progress_m[ 70]='echo -n "/"'
   T85progress_m[ 80]='printf "%5i" $htmlfilecount'
   T85progress_m[ 90]='printf " -  "'
   T85progress_m[100]='printf "%15s" ":$filename:"'
   T85progress_m[110]='printf " ==="'
#  T85progress_m[120]='echo -n "  - $finishtime"'
#  T85progress_m[130]='echo -n " $finishtimeglobal"'
   T85progress_m[990]='printf "\r"'
fi

if [[ ${#T85progress_h[@]} = 0 ]]; then
   T85progress_h[ 00]='printf "$now"'
   T85progress_h[ 10]='printf " v:Verbose'
   T85progress_h[ 20]='printf " m:Minimal'
   T85progress_h[ 30]='printf " s:standard'
   T85progress_h[ 40]='printf " %5ims" $msecsdiff'
   T85progress_h[ 60]='printf " %5i" $ifetched'
   T85progress_h[ 70]='echo -n "/"'
   T85progress_h[ 80]='printf "%5i" $htmlfilecount'
   T85progress_h[990]='printf "\r"'
fi

if [[ ${#T85progress_v[@]} = 0 ]]; then
   T85progress_v[ 00]='printf "$now"'
   T85progress_v[ 10]='printf " === %3i %% ===" $percent'
   T85progress_v[ 20]='printf " %4is" $secsdiff'
   T85progress_v[ 30]='printf " %5ims" $msecsdiff'
   T85progress_v[ 40]='printf " %5is" $secsused'
   T85progress_v[ 50]='printf " %5is" $secseta'
   T85progress_v[ 60]='printf " %5i" $ifetched'
   T85progress_v[ 70]='echo -n "/"'
   T85progress_v[ 80]='printf "%5i" $htmlfilecount'
   T85progress_v[ 90]='printf " -  "'
   T85progress_v[100]='printf "%15s" ":$filename:"'
   T85progress_v[110]='printf " ==="'
   T85progress_v[120]='echo -n "  - $finishtime"'
   T85progress_v[130]='echo -n " $finishtimeglobal"'
   T85progress_v[990]='printf "\n"'
fi

[ -z "$T85progressarr" ] && T85progressarr="T85progress"
#eval progressarr="(\"\${T85progressarr[@]}\")"

# http://ahmed.amayem.com/bash-arrays-3-different-methods-for-copying-an-array/
T85copyprogressarr() {
  # copy sparse array name $T85progressarr into array progressarr
  unset progressarr
  eval "progressarrindices=\${!$T85progressarr[@]}"
  for index in $progressarrindices;
  do
    eval "progressarr[\$index]=\"\${$T85progressarr[\$index]}\""
  done;
}

T85copyprogressarr

# local destination directory for the photos and videos from the camera
# needs to be existing
[ -z "$T85destdir" ] && T85destdir=.

now=$(date +"%Y%m%d_%H%M%S")

# After the files were read from the camera, they shall be moved to which
# place if any files are in $destdir before the run, the script will do 
# nothing but move them there

# if you don't want to move them to nasdir, you may leave nasdir identical to destdir
# Then the files will be moved to subdirectories
#
# if you don't want to move them after they got moved to nasdir, you may use different dirs for ever run
# for instance by using the date as dirname
# T85nasdir=$T85destdir/$now
#
[ -z "$T85nasdir" ] && T85nasdir="$T85destdir"

#
# To not set a WLAN, you may use NUL als Name for that WLAN.

# This WLAN needs to be active to receive the data from the camera
# You may enter the complete WLAN name if known but partial string will do
[ -z "$T85camWLAN" ] && T85camWLAN=Campark-

# The password for the T85WLAN
[ -z "${T85camWLANPWD+x}" ] && T85camWLANPWD=12345678

# This WLAN must be active to move the files to the NAS
# If it is not needed (eg using LAN connection) you may set it
# identical to the T85WLAN but  shouldn't forget to turn WLAN 
# off after using the batch to trigger timeout of the T85 WLAN
# T85stdWLAN=$T85camWLAN
[ -z "$T85stdWLAN" ] && T85stdWLAN="*keepWLANasis"

# The password for the stdWLAN
# if not set, it will be asked if not known to ubuntu
# recommendation: Let it empty
[ -z "${T85stdWLANPWD+x}" ] && T85stdWLANPWD=

[ -z "${T85exifren_runfile}" ] && T85exifren_runfile="$T85nasdir/T85exifren_runfile"

[ -f "${T85exifren_runfile}" ] && T85exifren_runagereal=$(( ($(date +%s) - $(date -r $T85exifren_runfile +%s) )/60/60 )) || T85exifren_runagereal=9999999

# if T85exifrun told in the last 48 hours that it is running, we may behave different (not anything implemented by now)
[ -z "${T85exifren_runagemax}" ] && T85exifren_runagemax=48

[ -z "${T85exifren_runok}" ] && T85exifren_runok='$T85exifren_runagereal < $T85exifren_runagemax'

[ -z "$T85get_runtimefmt" ] && T85get_runtimefmt="date +%F_%H-%M"

# This gives the name of nasdir for this run
T85get_runtimereal="$($T85get_runtimefmt)"
T85nasdestdir="$T85nasdir"

# End of individual settings

[ "$T85LOGDBG" ] && export T85DBG=1

[ -z "$T85DBGdir" ] && T85DBGdir="."

[ -z "$T85nasdir" ] && T85nasdir="$T85destdir"

[ -z "$T85getshdir" ] && export T85getshdir="$(dirname "${BASH_SOURCE[0]}")" 

[ -z "$T85camurl" ] && T85camurl="http://gateway_ip"

[ -z "$T85typename"    ] && T85typename=( "PHOTO" "MOVIE" )
[ -z "$T85typegetmsec" ] && T85typegetmsec=( 2500 10000 )
[ -z "$T85typeext"     ] && T85typeext=( "JPG" "MP4" )
[ -z "$T85typemask"    ] && T85typemask=( "*.JPG" "*.MP4" )
[ -z "$T85typeurlpost" ] && T85typeurlpost=( "/DCIM/PHOTO" "/DCIM/MOVIE" )

[ -z "$T85delmatch" ] && T85delmatch="?del=1"
[ -z "$T85delstring" ] && T85delstring="$T85delmatch"

if (( ${#T85typename[@]} != ${#T85typemask[@]} )); then
  echo Array types should have equal element count T85typename ${T85typename[@]} T85typemask ${T85typemask[@]}
fi

if (( ${#T85typename[@]} != ${#T85typeext[@]} )); then
  echo Array ext should have equal element count T85typename ${T85typename[@]} T85typeext ${T85typeext[@]}
fi

if (( ${#T85typename[@]} != ${#T85typeurlpost[@]} )); then
  echo Array urlpost should have equal element count T85typename ${T85typename[@]} T85typeurlpost ${T85typeurlpost[@]}
fi

[ -z "$T85getlog" ] && T85getlog="T85get.log"

[ -z "$T85userentrytime" ] && T85userentrytime=10

[ -z "$T85curltimeout" ] && T85curltimeout=10
[ -z "$T85curlspeedtime" ] && T85curlspeedtime=15
[ -z "$T85curlspeedlimit" ] && T85curlspeedlimit=500

if [ -z "$T85curlopt" ]; then
  T85curlopt="--connect-timeout $T85curltimeout --speed-time $T85curlspeedtime --speed-limit $T85curlspeedlimit"
  [ "$T85DBG" ] || T85curlopt="-s -S $T85curlopt"
fi

#findtypemask="${T85typemask[@]}"

filesfetched=0

filesiname="-type f"
for i in "${!T85typemask[@]}";
 do
  [[ $i > 0 ]] && filesiname="$filesiname -o"
  filesiname="$filesiname -iname ${T85typemask[$i]}"
 done

# Check if the variable is set, it may be set to empty but defined
# currently not working, seems to interact somehow with the redirection, gives error
#[ -z "${T85inhibitsleep+x}" ] && T85inhibitsleep="systemd-inhibit --no-pager"

[ -z "$T85idlegetcmd" ] && T85idlegetcmd="gsettings get org.gnome.desktop.session idle-delay"

[ -z "$T85idledefault" ] && T85idledefaultval="$($T85idlegetcmd)"
[ -z "$T85idlerun" ] && T85idlerun=0
[ "$T85idledefaulttime" ] && T85idledefault="${T85idledefaultval##* }"

[ -z "$T85idlesetcmd" ] && T85idleresetcmd="gsettings set org.gnome.desktop.session idle-delay $T85idlerun"
[ -z "$T85idleresetcmd" ] && T85idleresetcmd="gsettings set org.gnome.desktop.session idle-delay $T85idledefaulttime"

# Estimate speed: Count bytes, measure time
# just prepared: byte counting not implemented by now, not doing anything
#
# easy estimation inside a folder: Assume all files are equal size (usually they are at least similar enough)
# difficult way: Parse the sizes given in html, add them
# Parse future folder (MOVIE after PHOTO) would also be necessary to give an ETA
#
timestart=$(date +"%s%3N")

# Make it possible to pipe output via tee and still get the
# return code of first command in the pipe
set -o pipefail

# define functions

# logging
# Loglevels are strings. If the $1 string logcondition occurs in the current T85Loglevel, text $2 will be printed
# example: Loglevel "WLAN,Bluetooth" will print unconditional logs and Logs for WLAN and Bluetooth
T85log () {
  logcondition="$1"
  logstring="$2"

  # any condition and no DBG -> no log
  if [ -z "$T85DBG" ] && [[ ${#logcondition} > 0 ]]; then
    return 0
  fi

  # unconditional log with empty string always matches, so non-DBG is fine also
  if [[ ${#T85loglevel} == 0 ]] || [[ "$T85loglevel" == *"$logcondition"* ]]; then
    # beautify output: add a space
    if [[ ${#logcondition} > 0 ]]; then
      logcondition=" $logcondition"
    fi

    now=$(date +"%Y%m%d_%H%M%S")
    echo "$now$logcondition $logstring"
  fi
}

T85eval_key() {
  key="$1"
  result=0
        case "${key,,}" in
          "1")
            getrepeatsleep=$T85getrepeatfastsleep
            getrepeatfastend=$(date --date="+$T85getrepeatfasttime sec" +%s)
          ;;
          "0")
            getrepeatsleep=$T85getrepeatsleep
          ;;
          "d")
            [ "$T85DBG" ] && unset T85DBG || export T85DBG=1
            now=$(date +"%Y%m%d_%H%M%S")
            [ "$T85DBG" ] && echo "$now set T85DBG=$T85DBG" || echo "$now T85DBG unset, DBG inactive"
          ;;
          "")
            T85log "dbg" "no key pressed"
          ;;
        "d")
          [[ "$progressarr[990]" == 'printf "\n"' ]] || echo
          [ "$T85DBG" ] && unset T85DBG || T85DBG=1
          now=$(date +"%Y%m%d_%H%M%S")
          [ "$T85DBG" ] && echo "$now set T85DBG=$T85DBG" || echo "$now T85DBG unset, DBG inactive"
        ;;
        "s")
          [[ "$progressarr[990]" == 'printf "\n"' ]] || echo
          T85log "" "Standard print mode by key s"
          T85progressarr="T85progress_v"
          T85copyprogressarr
        ;;
        "m")
          [[ "$progressarr[990]" == 'printf "\n"' ]] || echo
          T85log "" "Minimal print mode by key m"
          T85progressarr="T85progress_m"
          T85copyprogressarr
        ;;
        "v")
          [[ "$progressarr[990]" == 'printf "\n"' ]] || echo
          T85log "" "Verbose mode by key v"
          T85progressarr="T85progress_v"
          T85copyprogressarr
        ;;
        "q")
            result=2;
        ;;
          *)
            result=1;
        ;;
      esac
  return $result
}
# loop over the output of an URL e.g. 192.168.8.120/DCIM/PHOTO
# search for ?del=1 and call T85getpic.sh with filename, link, removelink
# example line
# <tr><td><a href="/DCIM/PHOTO/IM_00672.JPG"><b>IM_00672.JPG</b></a><td align=right> 1.04 MB<td align=right>2020/07/06 18:47:44
# <td align=right><a href="/DCIM/PHOTO/IM_00672.JPG?del=1">Remove</a>
# only use the ?del=1 for searching
T85htmlloop () {
  local destdir htmlfilecount htmlfiletype nowstart iexist ifetched iseqfailed ifailed inotfound result 
  destdir="$1"
  htmlfilecount="$2"
  htmlfiletype="$3"

  htmlloopresult=0

  nowstart=$(date +"%Y%m%d_%H%M%S")
  unset hlepochstart

  # for the other file types to be downloaded estimate the duration
  msecsremaining=$(IFS=+; echo "$((${dataestmsec[*]}))")

  msecsused=$(IFS=+; echo "$((${datatimemsec[*]}))")

  [ "$T85DBG" ] && echo "dbg $nowstart T85htmlloop.sh start"
  iexist=0
  ifetched=0
  iseqfailed=0
  ifailed=0
  inotfound=0

  T85log "dbg" "T85camurl -$T85camurl-"

  [ -z "$T85fileretry" ] && T85fileretry=2
  [ -z "$T85maxseqfail" ] && T85maxseqfail=3
  [ -z "$T85maxfail" ] && T85maxfail=30
  [ -z "$T85maxfetch" ] && T85maxfetch=99999

  T85log "dbg" "T85delmatch -$T85delmatch- T85delstring -$T85delstring-"

  # htmlfile is local from calling function and is available here
  # read it into an array with one line each element

  readarray -t htmlfilearray < <(echo "$htmlfile")
  T85log "dbg" "count lines in htmlfile size ${#htmlfile} is ${#htmlfilearray[@]}"

  linenumber=0
  for i in "${!htmlfilearray[@]}"
  do

  [[ "$doquit" == "1" ]] && break

  line="${htmlfilearray[i]}"
  T85log "dbg" "parsing index $i line #$linenumber $line"

  # inside the table there is the first line with size and date of the file
  # example
  # <tr><td><a href="/DCIM/PHOTO/IM_00106.JPG"><b>IM_00106.JPG</b></a><td align=right> 3.55 MB<td align=right>2021/03/08 16:42:06
  # now parse the content
  if [[ "$line" =~ "<tr><td><a href=".* ]]; then
    unset origfilelink origfilename origsize origdate
    # first <td> is origfilelink and origfilename eg <tr><td><a href="/DCIM/PHOTO/IM_00113.JPG"><b>IM_00113.JPG</b>
    subline1="${line:17}"
    origfilelink=${subline1%%'"'*}
    if [[ "${subline1:${#origfilelink}:5}" == "\"><b>" ]]; then
      skipchars1=$(( ${#origfilelink} + 5 ))
      subline2="${subline1:$skipchars1}"
      origfilename=${subline2%%"<"*}
      if [[ "${subline2:${#origfilename}:24}" == "</b></a><td align=right>" ]]; then
        skipchars2=$(( ${#origfilename} + 24 ))
        subline3="${subline2:$skipchars2}"
        origsize=${subline3%%"<"*}
        if [[ "${subline3:${#origsize}:16}" == "<td align=right>" ]]; then
          skipchars3=$(( ${#origsize} + 16 ))
          subline4="${subline3:$skipchars3}"
          origdate="$subline4"
          T85log "Info" "parse line $linenumber successful, next file ist $origfilename"
        else
          T85log "" "parse line $linenumber start ok, but origsize is broken"
        fi
      else
        T85log "" "parse line $linenumber start ok, but origfilename is broken"
      fi
    else
      T85log "" "parse line $linenumber start ok, but origfilellink is broken"
    fi
    subline2="${subline1}"

    T85log "dbg" "parse line $linenumber result origdate $origdate origsize $origsize origfilename $origfilename origfilelink $origfilelink"

    # second <td align=right> is size
    # third <td align=right> is date
  else
  # now deal with the second line containing delstring
  # all characters of the line until ?del=1
    prefix=${line%%"$T85delmatch"*}
    pos=${#prefix}
    len=${#line}
    if [ $pos != $len ] ; then
      # there is ?del=1 in the line

# all chars of $prefix starting after last /
      filename=${prefix##*/}
# all chars of $prefix starting after last "
      filelink=${prefix##*\"}
# combine the link for remove, we know it was this way as we searched for this
      fileremove="$filelink$T85delstring"

      [[ -z "$hlepochstart" ]] && hlepochstart="$(date -u +%s%3N)"

      if [[ "$origfilelink" != "$filelink" ]]; then
        T85log "" "link in line with DEL is #$filelink#, but origfilelink from download-line differs, do not download origlink #$origfilelink# Name #$origfilename# Size #$origsize# Date #$origdate#"
        ((ifailed++))
        ((iseqfailed++))
      else
        if [[ -e $filename ]]; then
          echo "file -$filename- exists, do nothing - will not fetch existing file"
          ((iexist++))
        else
          [ "$T85DBG" ] && echo "dbg call T85getfile $pos $len $destdir/$filename $filelink $fileremove"
          for filetry in {1..$T85fileretry} ; do
            $T85getshdir/T85getfile.sh $destdir/$filename $T85camurl$filelink $T85camurl$fileremove
            result=$?
            [[ $result == 0 ]] && break
            [[ $result == 99 ]] && break   # too small filesize in html-file is not fixable by repeat
            T85log "" "Fetch failed result $result try $fileretry/$T85fileretry for -$filename-"
          done
          if [ $result != 0 ]; then
            ((ifailed++))
            ((iseqfailed++))
            T85log "" "T85getfile result $result for -$filename-"
          else
            ((ifetched++))
            iseqfailed=0
          fi
        fi
      fi
      read -rsn1 -t 0.00001 key
      T85eval_key "$key"
      result=$?
      if [[ "$result" == "2" ]]; then
        T85log "" "Quit requested by key"
        doquit=1
      fi

      now=$(date +"%Y%m%d_%H%M%S")
      percent=$(( 100*(ifetched+iexist+ifailed)/htmlfilecount ))
      hlepochnow="$(date -u +%s%3N)"
      [ -z "$msecslast" ] && msecslast=0 || msecslast=$msecsused
      msecsused=$(( hlepochnow-hlepochstart ))
      msecsdiff=$(( msecsused-msecslast ))
      [[ $ifetched > 0 ]] && msecsperfile=$(( msecsused/ifetched )) || msecsperfile=50000
      # Estimation of remaining time for current file type
      msecseta=$(( (msecsperfile*(htmlfilecount-iexist-ifetched-ifailed)) ))
      # remaining time estimation for current type plus estimation for other types gives estimation for global end
      msecsetaglobal=$(( msecseta + msecsremaining ))
      # estimation of duration for current type altogether
      msecsest=$(( msecsused+msecseta ))
      # estimation of duration for all file type: used time other types, used time current type,
      #                                           estimation for current tpye, estimation for other types
      msecsestglobal=$(( msecsused+msecsetaglobal+msecsused ))

# Copies of the msec variables for printing seconds
#      secsused=$(( msecsused/1000 ))
      secsused=$(bc <<< "scale=2; $msecsused/1000" ) # calculate with 2 decimal digits
      secsdiff=$(( msecsdiff/1000 ))
      secseta=$(( msecseta/1000 ))
      secslast=$(( msecslast/1000 ))
      secsetaglobal=$(( msecsetaglobal/1000 ))

      finishtime=$(date --date="$secseta sec" +"%H:%M:%S")
      finishtimeglobal=$(date --date="$secsetaglobal sec" +"%H:%M:%S")

      [ "$T85DBG" ] && echo "==============================================================================" >&2
      # progress line gets written to screen here
      for i in "${!progressarr[@]}"; do
        eval ${progressarr[$i]} >&2
      done
      [ "$T85DBG" ] && echo "==============================================================================" >&2
      [ "$T85DBG" ] && echo $now T85getfile $percent% $ifetched/$htmlfilecount "$filename"
      unset origfilelink origfilename origsize origdate
    else
# remove unset if there may be a line between original download link and line with del-link
      [ "$origfilelink" ] && printf "Missing del-Line for File Link #$origfilelink# Name #$origfilename# Size #$origsize# Date #$origdate#\ncur line is #$line#"
      unset origfilelink origfilename origsize origdate
      ((inotfound++))
    fi

    if (( $iseqfailed >= $T85maxseqfail )); then
      T85log "" "max seq fails accepted are $T85maxseqfail, we have $iseqfailed"
      break;
    fi
    if (( $ifailed >= $T85maxfail )); then
      T85log "" "max fails accepted are $T85maxfail, we have $ifailed"
      break;
    fi
    if (( $ifetched >= $T85maxfetch )); then
      T85log "" "max fetch accepted are $T85maxfetch, we have $ifetched"
      break;
    fi
    ((linenumber++))
  fi
  done

  nowend=$(date +"%Y%m%d_%H%M%S")
  echo $nowend' T85htmlloop end started: '$nowstart' - type '$htmlfiletype' - fetched: '$ifetched' - exist: '$iexist' failed: '$ifailed' nofileline: '$inotfound

  if (( $ifetched > 253 )); then
# return code of piped subshell is max 255
# below 254 the number of transferred files is given
# 255 means, count of files transmitted is the count that was given in htmlfile
# 254 means, it differs - pity that we can't easily transmit how much
    [[ $ifetched == $htmlfilecount ]] && result=255 || result=254
  else
    result=$ifetched
  fi

  T85log "Info" "T85htmlloop result $result fetched $ifetched"

  htmlloopresult=$ifetched
  return $result
}

# Get all files from one directory
# Parameter $1 is directory to fetch
# Parameter $2 is desired htmltype
T85getdir() {
  local destdir T85type filesfetched filecount htmlfile result now

  destdir="$T85destdir"
  T85type="$1"
  htmlfile="$2"
  delmatch="$3"

  filesfetched=0
  filecount=0

  T85log "Info" "T85getdir start Fetch data into $destdir for type $T85type"

  filecount=$(echo "$htmlfile" | grep "$delmatch" | wc -l)
  [ "$T85DBG" ] && echo "dbg $now HTML-page tells $filecount files of type $T85type"

  if (( $filecount > 0 )); then
    T85htmlloop $destdir $filecount $T85type
    filesfetched=$htmlloopresult

    T85log "Info" "T85getdir end htmlfile $T85type count given $filecount fetched $filesfetched"
  fi

  # return code can not give a result more than 255, use global variable
  if [[ $result == 0 ]]; then
    [[ $filesfetched > 0 ]] && result=0 || result=1
  fi
  getdirresult=$filesfetched

  [ "$T85DBG" ] && echo "dbg T85getdir result $result getdirresult $getdirresult"

  return $result
}

printsampleline() {
    progressbak="$T85progressarr"
    T85progressarr="$1"
    T85copyprogressarr
    percent=25
    secsused=2
    secseta=13
    secsest=$(( secsused+secseta ))
    finishtime=$(date --date="+$secseta sec" "+%H:%M:%S")
    ifetched=12234
    htmlfilecount=12267
    filename=IM_0001.JPG

    echo
    T85log "" "$2"
    echo -n "Sample progress format line according to $T85progressarr array, active $progressbak"
    [[ "$progressbak" == "$T85progressarr" ]] && echo "  (active)" || echo 
    for i in "${!progressarr[@]}"; do
      eval ${progressarr[$i]}
    done

    stringlength=0
    for i in "${!progressarr[@]}"; do
      erg=$(eval ${progressarr[$i]})
      printf "%2i is %2i chars: " "$i" "${#erg}"
      printf "   %-25s" "*$erg*"
      echo "  ${T85progress[$i]}"
      (( stringlength+=${#erg} ))
    done
    echo Length of string is $stringlength chars
    T85progressarr="$progressbak"
    T85copyprogressarr
}

rescode=0
if [[ "$T85cmd" != '-1' ]]; then
  T85log "Info" "cmd \"$T85cmd\" parameters \"$@\""
  case $T85cmd in
  -r)
# repeat until files exist, formerly T85getrepeat.sh
    [ "$T85getshdir" ] && cd $T85getshdir || cd "$(dirname "${BASH_SOURCE[0]}")"
    [ -z "$T85getrepeatsleep" ] && T85getrepeatsleep=30
    [ -z "$T85getrepeatfastsleep" ] && T85getrepeatfastsleep=1
    [ -z "$T85getrepeatfasttime" ] && T85getrepeatfasttime=60
    getrepeatsleep=$T85getrepeatsleep
    getresult=1
    while [[ $getresult != 0 ]]
    do
      echo start in $(pwd)
      if [ -d "$T85nasdir" ]; then
        [ "$T85DBG" ] && echo "dbg check if find $T85nasdir -maxdepth 1 $filesiname -quit"
        nasfiles="$(find $T85nasdir -maxdepth 1 $filesiname -quit)"
        if [ "$nasfiles" ]; then
           echo "$now Files exist in $T85nasdir, continue anyway? (Y/N)" 
           read -rsn1 -t $T85userentrytime contkey
           if [ "${contkey,,}" != "y" ]; then
             break
           fi
        fi
        T85log "Info" "not found: find $T85nasdir $filesiname -quit"
      else
        echo "nasdir $T85nasdir not available"
      fi
      $T85inhibitsleep $T85getshdir/T85get.sh -1 "$@"
      getresult=$?
      if [[ $getresult != 0 ]]; then
        [[ -z "$getrepeatfastend" ]] && getrepeatfastend=$(date --date="+1 day" +%s)
        if (( $getrepeatfastend < $(date +%s) )); then
          [ "$T85DBG" ] && echo "dbg $(date +%Y%m%d_%H%M%S) getrepeat fast time ends now after $T85getrepeatfasttime seconds"
          getrepeatsleep=$T85getrepeatsleep
        fi
        T85log "" "T85get result $getresult, sleep $getrepeatsleep (space/return to repeat, 1 fast, 0 default $T85getrepeatsleep, d debug toggle, other key to exit)"
        read -rsn1 -t $getrepeatsleep key
        T85eval_key "$key"
        result=$?
        [[ "$result" == "1" ]] && break
      fi
    done
    if [ ! "$key" = "" ]; then
     echo key $key pressed
    else
      nasfiles="$(find $T85nasdir -maxdepth 1 $filesiname -quit)"
      nasresult=$?
      now=$(date +"%Y%m%d_%H%M%S")
      [ "$nasfiles" ] && echo "$now now is ${T85typemask[@]} in $T85nasdir, find result $nasresult" || echo "$now No files fetched from camera, seems empty (result find $nasresult)"
    fi
  ;;
  -f)
# cmd -f: Sample for format line
          printsampleline "T85progress" "Standard print mode by key s"

          printsampleline "T85progress_m" "Minimal print mode by key m"

          printsampleline "T85progress_v" "Verbose mode by key v"
    ;;
  *)
    for s in "${usage[@]}"; do
      echo "$s"
    done
  ;;
  esac
else
  T85log "" "Start T85get.sh"
  T85log "Info" "start T85get.log active T85DBG='$T85DBG'"

  [[ "$T85LOGDBG" ]] && echo "$now start T85get.log T85DBG='$T85DBG' active T85LOGDBG='$T85LOGDBG'" | tee -a $T85getlog

  T85stdWLANon=$(nmcli net)
  [[ "$T85stdWLANon" != "enabled" ]] && nmcli net on

  filesfetched=$(find $T85destdir ${filesiname} | wc -l)
  if [[ $filesfetched = 0 ]]; then
    if [[ "$T85stdWLAN" == "*keepWLANasis" ]]; then
      T85stdWLANuuid=$(nmcli -g uuid connection show --active)
      T85stdWLANprofile=$(nmcli -g name,type connection show --active | grep wireless | head -n 1 | cut -d: -f1)
      T85stdWLAN=$(nmcli -g 802-11-wireless.ssid connection show "$T85stdWLANprofile")
      T85log "" "after fetch enable current WLAN -$T85stdWLAN- (uuid -$T85stdWLANuuid- profile -$T85stdWLANprofile-)"
      [[ "${T85stdWLAN:0:1}" == "*" ]] && T85stdWLAN="*$T85stdWLAN"
    else
        T85stdWLANon="enabled"
    fi
    [[ "$T85DBG" ]] && iw dev
    $T85getshdir/T85enablewlan.sh ON
    result=$?
    if [ $result != 0 ]; then
      T85log "" "T85enablewlan.sh ON result $result, exiting"
      exit $result
    fi
    [[ "$T85DBG" ]] && iw dev
        
    $T85getshdir/T85setwlan.sh "$T85camWLAN" T85camWLAN "$T85camWLANPWD"
    result=$?
    if [ $result != 0 ]; then
      T85log "" "T85setwlan.sh T85camWLAN -$T85camWLAN- result $result, exiting"
      exit $result
    fi
    [[ "$T85DBG" ]] && iw dev

    if [ "$T85camurl" != "${T85camurl/gateway_ip/gip}" ]; then
      wlandevice=$(nmcli dev | grep -i $T85camWLAN | head -n 1 | cut -d " " -f1)
      wlandeviceshowall=$(nmcli dev show)
      wlandeviceshowstart=$(echo $wlandeviceshowall | grep -n DEVICE | grep "$wlandevice" | head -n 1 | cut -d ":" -f1)
      wlangateway=$(echo "$wlandeviceshowall" | tail -n +$wlandeviceshowstart | grep "IP4.GATEWAY:" | head -n 1 | sed 's/.* //')
      [ "$T85DBG" ] && echo "dbg gateway $wlangateway"
      T85camurl=${T85camurl/gateway_ip/$wlangateway}
    fi

    if [[ "$wlangateway" != "--" ]]; then
      if [[ -z "$T85routeprohibit" ]]; then
        if (( $EUID = 0 )); then
          T85log "" "route add -net ${wlangateway%.*}.0/24 dev $wlandevice"
          route add -net "${wlangateway%.*}.0/24" dev "$wlandevice"
        else
          T85log "" "For route add please run as root"
        fi
      fi

      [ "$T85DBG" ] && echo "dbg camurl $T85camurl"
      export T85camurl
      export T85camWLAN

      [ -z "$T85camurlcheck" ] && T85camurlcheck="http://192.168.8.120"
      if [[ "$T85camurlcheck" != "$T85camurl" ]]; then
        T85log "" "!URL! expected camurl $T85camurlcheck, instead it is $T85camurl"
      fi

      # see https://stackoverflow.com/questions/17840322/how-to-undo-exec-dev-null-in-bash
      STDOUT=`readlink -f /proc/$$/fd/1`
      STDERR=`readlink -f /proc/$$/fd/2`

      # see https://stackoverflow.com/questions/3173131/redirect-copy-of-stdout-to-log-file-from-within-bash-script-itself
      exec >  >(tee -ia $T85getlog)

      if [ "$T85LOGDBG" ] ; then
        # logging progress usually only shown on screen to logfile also
        exec 2> >(tee -ia $T85getlog >&2)
      fi

      T85log "Info" "Getting information about existing files"
      datainfo=""
      for i in ${!T85typename[@]}; do
        datahtmlfile[$i]=$(curl $T85curlopt $T85camurl${T85typeurlpost[$i]})
        dataresult[$i]=$?
        datafilecount[$i]=0
        now=$(date +"%Y%m%d_%H%M%S")
        if [ ${dataresult[$i]} == 0 ]; then
          [[ "$T85DBG" ]] && echo "dbg $now curl datahtmlfile type ${T85typename[$i]} result OK"
          [[ "$T85DBGfile" ]] && echo "${datahtmlfile[$i]}" >"$T85DBGdir/$now-${T85typename[$i]}.html"
          datafilecount[$i]=$(echo "${datahtmlfile[$i]}" | grep "$T85delmatch" | wc -l)
          T85log "Info" "HTML-page shows ${datafilecount[$i]} files of type ${T85typename[$i]} delmatch $T85delmatch"
          datainfo="$datainfo ${T85typename[$i]}:${datafilecount[$i]}"
          [ -z "${T85typegetmsec[$i]}" ] && T85typegetmsec[$i]=4000
          # the estimation for the time to be used to fetch the files
          dataestmsec[$i]=$(( ${datafilecount[$i]} * ${T85typegetmsec[$i]} ))
        else
          echo "$now htmlfile ${T85typename[$i]} curl $T85camurl${T85typeurlpost[$i]} ERROR Ergebnis ${dataresult[$i]}"
        fi
      done
      T85log "Info" "Got info $datainfo"

      $T85idlesetcmd

      datarestext=""
      for i in "${!T85typename[@]}"; do
        if [[ ${datafilecount[$i]} > 0 ]]; then
          # for already fetched data the estimation for the time to fetch is 0, so sum of the array always makes sense
          dataestmsec[$i]=0
          datatype="${T85typename[$i]}"
          timestart=$(date +%s%3N)
          T85log "Info" "get type $datatype result OK"
          T85getdir "$datatype" "${datahtmlfile[$i]}" "$T85delmatch"
          datatimestart[$i]=$timestart
          dataresult[$i]=$?
          datafetched[$i]=$getdirresult
          (( filesfetched+=$getdirresult ))
          datatimeend[$i]=$(date +%s%3N)
          datatimemsec[$i]=$(( ${datatimeend[$i]} - ${datatimestart[$i]} ))
          [[ "$T85DBG" ]] && echo "dbg $now time used datatimemsec[$i] (started ${datatimestart[$i]} ended ${datatimeend[$i]})"
          datarestext="$datarestext $getdirresult $datatype"
       else
          [[ "$T85DBG" ]] && echo "dbg $now not fetching type ${T85typename[$i]} - not available"
       fi
      done

      if [ "$T85LOGDBG" ] ; then
        exec 1>$STDOUT 2>$STDERR
      else
        exec 1>$STDOUT
      fi

      timeend=$(date +%s%3N)

      if [[ $filesfetched > 0 ]]; then
        now=$(date +"%Y%m%d_%H%M%S")
        echo $now Got $filesfetched files, detail $datarestext | tee -a $T85getlog
      else
        T85log "" "Got nothing!"
      fi
    else
      T85log "" "WLAN not ok, bad gateway"
    fi
    $T85getshdir/T85enablewlan.sh OFF
    [[ "$T85DBG" ]] && iw dev
    if [[ "$T85stdWLANon" == "enabled" && -z "$T85stdWLANturnoff" ]]; then
      [[ "${T85stdWLAN:0:2}" == "**" ]] && T85stdWLAN="${T85stdWLAN:1}"
      $T85getshdir/T85setwlan.sh $T85stdWLAN stdWLAN $T85stdWLANPWD
    else
      # WLAN was disabled before, and *keepWLANasis set
      nmcli net off
    fi
    result=$?
    if [ $result != 0 ]; then
      T85log "" "T85setwlan.sh stdWLAN error: result $result"
      exit $result
    fi
    [[ "$T85DBG" ]] && iw dev
  else
    now=$(date +"%Y%m%d_%H%M%S")
    echo $now destdir $T85destdir is not empty, only moving $filesfetched files | tee -a $T85getlog
    ls -l $T85destdir | tee -a $T85getlog
  fi

  if [[ $filesfetched > 0 ]]; then
    if [[ "$T85nasdir" != "$T85destdir" ]]; then
      #  create dir if not existing
      [ ! -d "$T85nasdir" ] && mkdir "$T85nasdir"
      [ ! -d "$T85nasdestdir" ] && mkdir "$T85nasdestdir"
      for i in "${!T85typemask[@]}";
      do
        T85log "Info" "move files typemask ${T85typemask[$i]} to $T85nasdestdir"
        mv -n -v $T85destdir/${T85typemask[$i]} $T85nasdestdir
      done
    fi
    if [[ "$T85exifrencall" ]]; then
      $T85exifrencall 
    else
      T85log "Info" "T85exifrencall is not set"
    fi
  else
    T85log "" "No files to move from destdir $T85destdir to NAS $T85nasdir"
    [[ "$datainfo" ]] || rescode=1
  fi
fi
$T85idleresetcmd
exit $rescode

